<?php namespace EwTools\Admin\ProjectPluginPage;

use EwTools\Interfaces\PluginPageInterface;
use EwTools\Interfaces\SettingsPageInterface;
use EwTools\Support\Collection;
use EwTools\Support\Str;
use EwTools\Twig\View;

/**
 * TabsPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin
 * @since   EwTools\Admin\TabsPage 0.1
 */
abstract class TabsPage implements PluginPageInterface {
    
    /**
     * Path to the twig template base
     */
    protected static $template_path = '@tools/admin/';
    
    /**
     * @var string
     */
    protected static $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains messages from the tab-pages
     *
     * @var array
     */
    protected $messages;
    
    /**
     * Optional description of tabs page
     *
     * @var string
     */
    private $description;
    
    /**
     * Optional headline of tabs page
     *
     * @var
     */
    private $headline;
    
    /**
     * Contains the pages added as tab to the plugins settings-page
     *
     * @var SettingsPageInterface[]
     */
    private $settings_pages = [];
    
    /**
     * Contains the settings-page currently active
     *
     * @var SettingsPageInterface
     */
    private $current_page;
    
    /**
     * Function to set a description for the tabs page. Will be rendered above tabs
     *
     * @param string $description
     *
     * @since 0.1
     * @return self
     */
    protected function setDescription( $description ) {
        $this->description = $description;
        return $this;
    }
    
    /**
     * Function to set a description for the tabs page. Will be rendered above tabs
     *
     * @param string $headline
     *
     * @since 0.1
     * @return self
     */
    protected function setHeadline( $headline ) {
        $this->headline = $headline;
        return $this;
    }
    
    /**
     * Add a page to be displayed as a tab on the settings page
     *
     * @param SettingsPageInterface $page
     *
     * @since 0.1
     * @return $this
     */
    public function addSettingsTab( SettingsPageInterface $page ) {
        $this->settings_pages[ $page->getTabSlug() ] = $page;
        
        return $this;
    }
    
    /**
     * Use {add_submenu_page} to add the page to the plugin menu.
     *
     * @param string $parent_slug
     *
     * @since 0.1
     * @return string The menu hook_suffix
     */
    public function addToMenu( $parent_slug ) {
        $title = $this->getTitle();
        $hook  = add_submenu_page( $parent_slug, $title, $title, 'manage_options', $this->getPageSlug( $parent_slug ), [
            $this,
            'renderSettingsPage'
        ] );
        
        return add_action( "load-{$hook}", [ $this, 'onSettingsPageLoad' ] );
    }
    
    /**
     * Determine and fetch the current page based on the active tab
     *
     * @since 0.1
     * @return SettingsPageInterface
     */
    protected function getCurrentPage() {
        return $this->getSettingsTabs()->get( $this->getActiveTab() );
    }
    
    /**
     * Build page slug "admin.php/?page=[PageSlug]"
     *
     * @param $parent_slug
     *
     * @since 0.1
     * @return string
     */
    protected function getPageSlug( $parent_slug ) {
        return Str::slug( "{$parent_slug}-".$this->getTitle() );
    }
    
    /**
     * Fires whenever the settings page is loaded
     *
     * @since 0.1
     * @return void
     */
    public function onSettingsPageLoad() {
        $page = $this->getCurrentPage();
        if( $page instanceof SettingsPageInterface ) {
            $page->onPageLoad();
            $this->current_page = $page;
        }
    }
    
    /**
     * Display settings page
     */
    public function renderSettingsPage() {
        
        View::render( $this->getTemplate( 'tabs-page' ), [
            'tabs'         => $this->getTabs(),
            'headline'     => $this->headline?: $this->getTitle(),
            'description'  => $this->description?: '',
            'text_domain'  => static::$text_domain,
            'message'      => $this->current_page ? $this->getPageMessage( $this->current_page ) : '',
            'page_content' => $this->current_page ? $this->getPageContent( $this->current_page ) : ''
        ] );
    }
    
    /**
     * Collect data from each registered tab-page
     *
     * @since 0.1
     * @return array
     */
    protected function getTabs() {
        $active_tab = $this->getActiveTab();
        
        return array_map( function ( SettingsPageInterface $page ) use ( $active_tab ) {
            return [
                'slug'   => $page->getTabSlug(),
                'label'  => $page->getTabTitle(),
                'active' => $page->getTabSlug() === $active_tab,
                'url'    => add_query_arg( 'tab', $page->getTabSlug() )
            ];
        }, $this->settings_pages );
    }
    
    /**
     * Retrieve the tab-slug currently active on the settings page
     *
     * @since 0.1
     * @return string
     */
    public function getActiveTab() {
        
        if( isset( $_GET[ 'tab' ] ) ) {
            return $_GET[ 'tab' ];
        }
        $settings_pages = $this->getSettingsTabs();
        if( ! $settings_pages->isEmpty() ) {
            return $settings_pages->first()->getTabSlug();
        }
        
        return $this->getSlug();
    }
    
    /**
     * Retrieve a collection of the pages displayed ass tabs on the settings page
     *
     * @since 0.1
     * @return Collection
     */
    public function getSettingsTabs() {
        return collect( $this->settings_pages );
    }
    
    /**
     * Retrieve content from the provided page
     *
     * @param SettingsPageInterface $page
     *
     * @since 0.1
     * @return string
     */
    public function getPageContent( SettingsPageInterface $page ) {
        return $page->pageContent();
    }
    
    /**
     * Retrieve any messages from the provided page
     *
     * @since 0.1
     *
     * @param SettingsPageInterface $page
     *
     * @return string
     */
    protected function getPageMessage( SettingsPageInterface $page ) {
        return $page->getMessage();
    }
    
    /**
     * Retrieve a template from configured path
     *
     * @param $template
     *
     * @since 0.1
     * @return string
     */
    protected function getTemplate( $template ) {
        return Str::finish( static::$template_path, "/{$template}" );
    }
}