<?php
/**
 * Abstract class for handling widgets in Wordpress
 */
namespace EwTools\Admin;

use EwTools\Support\Str;
use \WP_Widget;
use EwTools\Twig\View;

/**
 * Widget
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin
 * @since   EwTools\Admin\Widget 0.1
 */
abstract class Widget extends WP_Widget {
    
    /**
     * Widget Text domain
     *
     * @var      string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains the prefix for the html classes used by the widget
     *
     * @var string
     */
    protected $widget_class = 'ew-widget';
    
    /**
     * Contains the fields, and its default-values, used in the form
     *
     * @var array
     */
    protected $fields = [];
    
    /**
     * Contains a list of rules for the form fields used for validation
     *
     * @var array
     */
    protected $rules = [];
    
    /**
     * Contains the path to where the templates are stored
     *
     * @var string
     */
    protected $template_path = '@tools/widgets/';
    
    /**
     * Contains the auto generated view data
     *
     * @var array
     */
    protected $view_data = [];
    
    /**
     * Contains the widgets instance number
     *
     * @var string
     */
    private $widget_id;
    
    /**
     * Contains the widgets untranslated name
     *
     * @var string
     */
    protected $widget_name = 'Widget Name';
    
    /**
     * Contains the widget untranslated description
     *
     * @var string
     */
    protected $widget_description = 'Widget Description';
    
    /**
     * Contains the widgets version
     *
     * @var string
     */
    protected $widget_version = '0.1';
    
    /**
     * Contains the version a widget will get if no version exist
     *
     * @var string
     */
    private $default_version = '0.1';
    
    /**
     * Widget constructor.
     *
     * @param string $id
     * @param array  $widget_options
     * @param array  $control_options
     *
     * @since 0.1
     */
    public function __construct( $id, $widget_options = [], $control_options = [] ) {
        
        $widget_options = array_replace_recursive( [
            'classname'   => $id,
            'description' => $this->widget_description
        ], $widget_options );
        
        $this->widget_id    = $id;
        $this->widget_class = $widget_options[ 'classname' ];
        
        parent::__construct( $id, $this->widget_name, $widget_options, $control_options );
    }
    
    // Public Functions
    // ======================================================
    
    /**
     * Back-end widget form.
     *
     * @see   WP_Widget::form()
     * @uses  wp_parse_args()
     * @uses  WP_Widget::get_field_id
     *
     * @param array $instance Previously saved values from database.
     *
     * @since 0.1
     * @return void
     */
    public function form( $instance ) {
        
        $is_new_instance = empty( $instance );
        
        // Add current widget version to new instances of the widget
        if( $is_new_instance ) {
            $instance[ 'widget_version' ] = $this->widget_version;
        }
        
        // Add default fields
        $this->fields = array_replace_recursive( [
            'widget_version' => $this->default_version
        ], $this->fields );
        
        $instance = wp_parse_args( (array)$instance, $this->fields );
        
        $this->view_data = [
            'widget' => [
                'name'         => $this->name,
                'class_prefix' => $this->widget_class . '-form',
                'id_prefix'    => $this->get_field_id( '' ),
                'number'       => $this->number,
                'version'      => $this->widget_version,
                'up_to_date'   => $this->isUpToDate( $instance )
            ]
        ];
        
        //update version on instance to be updated on save
        $instance[ 'widget_version' ] = $this->widget_version;
        
        $template_array = array_replace( $this->view_data, $this->setupFields( $instance ) );
        
        // Send the instance to the correct function
        if( $is_new_instance ) {
            $template_array = $this->onCreate( $template_array );
        }
        
        // Add title unless it's being used as a field from the widget
        if( ! isset( $this->fields[ 'title' ] ) ) {
            $template_array[ 'title' ] = [
                'id'    => $this->get_field_id( 'title' ),
                'name'  => '', // No name to make sure it isn't being save
                'value' => isset( $instance[ 'board_widget_name' ] ) ? $instance[ 'board_widget_name' ] : '',
            ];
        }
        
        $this->edit( $template_array );
    }
    
    /**
     * Sanitize widget form values as they are saved.
     *
     * @see   WP_Widget::update()
     *
     * @param array $new_instance Values just sent to be saved.
     * @param array $old_instance Previously saved values from database.
     *
     * @since 0.1
     * @return array Updated safe values to be saved.
     */
    public function update( $new_instance, $old_instance ) {
        
        // Update the form if the new data is valid
        if( $this->validate( $new_instance, $this->rules ) ) {
            
            $new_instance = array_replace_recursive( $this->fields, $new_instance );
            
            // Store if new form or change existing form
            if( empty( $old_instance ) ) {
                return $this->onStore( $new_instance );
            } else {
                return $this->onChange( $new_instance );
            }
            
        }
    }
    
    /**
     * Front-end display of widget.
     *
     * @see   WP_Widget::widget()
     * @uses  wp_parse_args()
     *
     * @param array $args     Widget arguments.
     * @param array $instance Saved values from database.
     *
     * @since 0.1
     * @return void
     */
    public function widget( $args, $instance ) {
        $instance = wp_parse_args( (array)$instance, $this->fields );
        
        $this->view_data = [
            'widget_args'         => $args,
            'widget_name'         => $this->name,
            'widget_class_prefix' => $this->widget_class,
            'widget_nr'           => $this->number
        ];
        
        $this->show( array_replace_recursive( $this->view_data, $instance ) );
    }
    
    /**
     * Function to render widget content in EveryBoard.
     *
     * @param $instance
     *
     * @since 0.1
     * @return void
     */
    public function widget_board( $instance ) {
        print $this->boardWidgetContent( $instance );
    }
    
    // Protected Functions: Can be used by inherited classes
    // ======================================================
    
    /**
     * Setup the desired content of the widget to be presented in Everyboard admin
     *
     * @param $instance
     *
     * @since 0.1
     * @return string
     */
    protected function boardWidgetContent( $instance ) {
        return "<p>{$this->widget_description}</p>";
    }
    
    /**
     * Return an array with the id and name from a multi valued field for the form input
     *
     * @param $field
     * @param $instance
     *
     * @since 0.1
     * @return array
     */
    protected function getFieldMultiValues( $field, $instance ) {
        $return_val = [];
        foreach ( $instance as $i => $value ) {
            
            if( ! is_array( $value ) ) {
                return $this->getFieldValues( "{$field}[]", $instance );
            }
            
            foreach ( $value as $settings_name => $settings_value ) {
                $return_val[ $i ][ $settings_name ] = $this->getFieldValues( "{$field}[{$i}][{$settings_name}]", $instance[ $i ][ $settings_name ] );
            }
        }
        
        return $return_val;
    }
    
    /**
     * Return an array with the id and name for the form input
     *
     * @uses  WP_Widget::get_field_id()
     * @uses  WP_Widget::get_field_name()
     *
     * @param $field
     * @param $value
     *
     * @since 0.1
     * @return array
     */
    protected function getFieldValues( $field, $value ) {
        
        return [
            'id'    => $this->get_field_id( $field ),
            'name'  => $this->get_field_name( $field ),
            'value' => $value
        ];
    }
    
    /**
     * Fires on saving updated widget
     *
     * @param array $instance
     *
     * @since 0.1
     * @return array $new_instance The form data to be saved
     */
    protected function onChange( $instance ) {
        return $instance;
    }
    
    /**
     * Fires on creating new widget
     *
     * @param array $instance
     *
     * @since 0.1
     * @return string The widget form to be rendered
     */
    protected function onCreate( $instance ) {
        return $instance;
    }
    
    /**
     * Fires on saving new widget
     *
     * @param array $new_instance
     *
     * @since 0.1
     * @return array $new_instance The form data to be saved
     */
    protected function onStore( $new_instance ) {
        return $new_instance;
    }
    
    /**
     * Function to setup field-values for the view
     *
     * @param $instance
     *
     * @since 0.1
     * @return array
     */
    protected function setupFields( $instance ) {
        $fields_values = [];
        
        foreach ( $this->fields as $field => $value ) {
            $fields_values[ $field ] = is_array( $value ) ? $this->getFieldMultiValues( $field, $instance[ $field ] ) : $this->getFieldValues( $field, $instance[ $field ] );
        }
        
        return $fields_values;
    }
    
    /**
     * Extract the data from the widgets dock-header
     * and save into properties to be used when instantiating the widget
     *
     * @uses  get_file_data()
     *
     * @param string $file The widget file with the docBlock
     *
     * @since 0.1
     * @return array the extracted data
     */
    protected function setWidgetData( $file ) {
        
        $widget_data = get_file_data( $file, [
            'name'        => 'Plugin Name',
            'description' => 'Description',
            'version'     => 'Version'
        ] );
        
        $this->widget_name        = $this->translate( $widget_data[ 'name' ] );
        $this->widget_description = $this->translate( $widget_data[ 'description' ] );
        $this->widget_version     = $this->translate( $widget_data[ 'version' ] );
        
        return $widget_data;
    }
    
    /**
     * Function to translate text using the widgets text-domain
     *
     * @uses  __()
     *
     * @param $text
     *
     * @since 0.1
     * @return mixed
     */
    protected function translate( $text ) {
        return __( $text, $this->text_domain );
    }
    
    /**
     * Function for validating the form
     *
     * @param array $new_instance
     * @param array $rules
     *
     * @since 0.1
     * @return bool
     */
    protected function validate( array $new_instance, array $rules ) {
        return true;
    }
    
    /**
     * Function to render a template from the set path
     *
     * @param string $template
     * @param array  $template_array
     *
     * @since 0.1
     * @return void
     */
    protected function view( $template = 'form', $template_array ) {
        View::render( $this->template_path . '/' . $template, $template_array );
    }
    
    /**
     * Function to compare the version of the widget and the saved instance
     *
     * @param $instance
     *
     * @since 0.1
     * @return bool
     */
    private function isUpToDate( $instance ) {
        return version_compare( $instance[ 'widget_version' ], $this->widget_version ) >= 0;
    }
    
    /**
     * Fires when rendering the widget form
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    abstract protected function edit( $view_data );
    
    /**
     * Fires when rendering widget
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    abstract protected function show( $view_data );
    
    /**
     * Function used to register widget in Wordpress
     *
     * @uses  add_action('widgets_init')
     * @uses  register_widget()
     * @since 0.1
     * @return void
     */
    public static function register() {
        add_action( 'widgets_init', function () {
            register_widget( get_called_class() );
        } );
    }
}
